// Copyright 2004, 2008, 2011 Jon Dart. All Rights Reserved.
//
// This module interfaces with bitbases and other in-memory scoring
// tables.

#include "bitprobe.h"
#include "globals.h"
#include "tbprobe.h"
#include <fstream>

static int getIndex(Square pawnSq, Square kingSq, Square oppKingSq) {
  return ((((pawnSq-8)<<6) | kingSq)<<6)| oppKingSq;
}

int lookupBitbase(const byte *base, int index) {
   ASSERT(index<24576*8);
   return (base[index/8] & (1<<(index%8))) != 0;
}

int lookupBitbase(Square whiteK, Square whiteP, Square blackK,
   ColorType strongerSide, ColorType sideToMove) {
   if (strongerSide == White) {
      if (sideToMove == White)
        return lookupBitbase(baseKPKW,getIndex(whiteP,whiteK,blackK));
      else
        return lookupBitbase(baseKPKB,getIndex(whiteP,whiteK,blackK));
   }
   else {
      if (sideToMove == White)
        return lookupBitbase(baseKPKB,getIndex(Flip[whiteP],Flip[whiteK],Flip[blackK]));
      else
        return lookupBitbase(baseKPKW,getIndex(Flip[whiteP],Flip[whiteK],Flip[blackK]));
   }
}

int lookupBitbase(const Board &board) {
   const Material &wMat = board.getMaterial(White);
   const Material &bMat = board.getMaterial(Black);
   int index;
   if (wMat.materialLevel() > 0 || 
       bMat.materialLevel() > 0) return NOHIT;
   if (wMat.infobits() == Material::KP &&
       bMat.kingOnly()) {
     Square pawnSq = board.pawn_bits[White].firstOne();
     index = getIndex(pawnSq,board.kingSquare(White),board.kingSquare(Black));
     if (board.sideToMove() == Black)
        return lookupBitbase(baseKPKB,index);
     else
        return lookupBitbase(baseKPKW,index);
   }
   else if (bMat.infobits() == Material::KP &&
       wMat.kingOnly()) {
     Square pawnSq = board.pawn_bits[Black].firstOne();
     index = getIndex(pawnSq,board.kingSquare(Black),board.kingSquare(White));
     if (board.sideToMove() == Black)
        return lookupBitbase(baseKPKW,index);
     else
        return lookupBitbase(baseKPKB,index);
   }
   return NOHIT; // shouldn't get here
}
